@@ -101,8 +101,8 @@ QRect rotateWithTf(int rotation, KisPaintDeviceSP dev,
     KisPaintDeviceSP tmp = new KisPaintDevice(dev->colorSpace());
     tmp->prepareClone(dev);
 
-    KisRandomAccessorSP devAcc = dev->createRandomAccessorNG(0, 0);
-    KisRandomAccessorSP tmpAcc = tmp->createRandomAccessorNG(0, 0);
+    KisRandomAccessorSP devAcc = dev->createRandomAccessorNG();
+    KisRandomAccessorSP tmpAcc = tmp->createRandomAccessorNG();
     KisProgressUpdateHelper progressHelper(progressUpdater, portion, r.height());
 
     QTransform tf;
@@ -216,7 +216,7 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
         KisFilterWeightsApplicator::LinePos dstPos;
         KisFilterWeightsApplicator::LinePos srcPos(srcStart, srcLen);
 
-        dstPos = applicator.processLine<T>(srcPos, i, &buf, filterStrategy->support());
+        dstPos = applicator.processLine<T>(srcPos, i, &buf, filterStrategy->support(buf.weightsPositionScale().toFloat()));
         dstBounds.unite(dstPos);
 
         progressHelper.step();
@@ -241,10 +241,15 @@ bool KisTransformWorker::runPartial(const QRect &processRect)
 {
     /* Check for nonsense and let the user know, this helps debugging.
     Otherwise the program will crash at a later point, in a very obscure way, probably by division by zero */
-    Q_ASSERT_X(m_xscale != 0, "KisTransformer::run() validation step", "xscale == 0");
-    Q_ASSERT_X(m_yscale != 0, "KisTransformer::run() validation step", "yscale == 0");
+    /* Note that because KisFilterWeightsBuffer uses KisFixedPoint, we need to check the fixed point as well
+     * since it has different precision than qreal or double */
+    KisFixedPoint m_xscale_fixedPoint = KisFixedPoint(m_xscale);
+    KisFixedPoint m_yscale_fixedPoint = KisFixedPoint(m_yscale);
+    Q_ASSERT_X(m_xscale != 0 && m_xscale_fixedPoint != 0, "KisTransformer::run() validation step", "xscale == 0");
+    Q_ASSERT_X(m_yscale != 0 && m_yscale_fixedPoint != 0, "KisTransformer::run() validation step", "yscale == 0");
+
     // Fallback safety line in case Krita is compiled without ASSERTS
-    if (m_xscale == 0 || m_yscale == 0) return false;
+    if (m_xscale == 0 || m_yscale == 0 || m_xscale_fixedPoint == 0 || m_yscale_fixedPoint == 0) return false;
 
     m_boundRect = processRect;
 
@@ -452,11 +457,16 @@ void mirror_impl(KisPaintDeviceSP dev, qreal axis, bool isHorizontal)
         rightEnd = mirrorRect.y() + mirrorRect.height();
     }
 
+    /**
+     * If the axis is not aligned, that is crosses some pixel cell, we should just skip this
+     * column and not process it. Actually, how can we mirror the central single-pixel column?
+     */
+    const bool axisNonAligned = qFloor(axis) < axis;
 
-    int leftCenterPoint = qFloor(axis) < axis ? qFloor(axis) : qFloor(axis);
+    int leftCenterPoint = qFloor(axis);
     int leftEnd = qMin(leftCenterPoint, rightEnd);
 
-    int rightCenterPoint = qFloor(axis) < axis ? qCeil(axis) : qFloor(axis);
+    int rightCenterPoint = axisNonAligned ? qCeil(axis) : qFloor(axis);
     int rightStart = qMax(rightCenterPoint, leftStart);
 
     int leftSize = qMax(0, leftEnd - leftStart);
@@ -476,8 +486,8 @@ void mirror_impl(KisPaintDeviceSP dev, qreal axis, bool isHorizontal)
     int initialRightCol = rightCenterPoint + maxDistanceToAxis - 1;
 
 
-    KisRandomAccessorSP leftIt = dev->createRandomAccessorNG(mirrorRect.x(), mirrorRect.y());
-    KisRandomAccessorSP rightIt = dev->createRandomAccessorNG(mirrorRect.x(), mirrorRect.y());
+    KisRandomAccessorSP leftIt = dev->createRandomAccessorNG();
+    KisRandomAccessorSP rightIt = dev->createRandomAccessorNG();
     const KoColor defaultPixelObject = dev->defaultPixel();
     const quint8 *defaultPixel = defaultPixelObject.data();
 
