@@ -259,10 +259,10 @@ void KisWelcomePageWidget::slotUpdateThemeColors()
     userCommunityLink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://krita-artists.org\">")
                                .append(i18n("User Community")).append("</a>"));
 
-    gettingStartedLink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://docs.krita.org/en/user_manual/getting_started.html?" + analyticsString + "getting-started" + "\">")
+    gettingStartedLink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://docs.krita.org/en/user_manual/getting_started.html\">")
                                 .append(i18n("Getting Started")).append("</a>"));
 
-    manualLink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://docs.krita.org?" + analyticsString + "documentation-site" + "\">")
+    manualLink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://docs.krita.org\">")
                         .append(i18n("User Manual")).append("</a>"));
 
     supportKritaLink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://krita.org/en/support-us/donations?" + analyticsString + "donations" + "\">")
@@ -271,10 +271,10 @@ void KisWelcomePageWidget::slotUpdateThemeColors()
     kritaWebsiteLink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://www.krita.org?" + analyticsString + "marketing-site" + "\">")
                               .append(i18n("Krita Website")).append("</a>"));
 
-    sourceCodeLink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://invent.kde.org/graphics/krita.git?" + analyticsString + "source-code" + "\">")
+    sourceCodeLink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://invent.kde.org/graphics/krita\">")
                             .append(i18n("Source Code")).append("</a>"));
 
-    poweredByKDELink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://userbase.kde.org/What_is_KDE?" + analyticsString + "what-is-kde" + "\">")
+    poweredByKDELink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://userbase.kde.org/What_is_KDE\">")
                               .append(i18n("Powered by KDE")).append("</a>"));
 
 
@@ -293,12 +293,20 @@ void KisWelcomePageWidget::slotUpdateThemeColors()
 void KisWelcomePageWidget::populateRecentDocuments()
 {
 
+    constexpr const int maxItemsCount = 5;
+
     m_recentFilesModel.clear(); // clear existing data before it gets re-populated
 
     // grab recent files data
-    int numRecentFiles = m_mainWindow->recentFilesUrls().length() > 5 ? 5 : m_mainWindow->recentFilesUrls().length(); // grab at most 5
+    int numRecentFiles = m_mainWindow->recentFilesUrls().length() > maxItemsCount ? maxItemsCount : m_mainWindow->recentFilesUrls().length(); // grab at most 5
     KisFileIconCreator iconCreator;
 
+    const int itemHeight = recentDocumentsListView->height() / maxItemsCount
+            - recentDocumentsListView->spacing() * 2;
+
+    QSize iconSize(itemHeight, itemHeight);
+
+
     for (int i = 0; i < numRecentFiles; i++ ) {
 
         QStandardItem *recentItem = new QStandardItem(1,2); // 1 row, 1 column
@@ -315,7 +323,7 @@ void KisWelcomePageWidget::populateRecentDocuments()
             recentItem->setIcon(m_thumbnailMap[recentFileUrlPath]);
         } else {
             QIcon icon;
-            bool success = iconCreator.createFileIcon(recentFileUrlPath, icon, devicePixelRatioF());
+            bool success = iconCreator.createFileIcon(recentFileUrlPath, icon, devicePixelRatioF(), iconSize);
             if (success) {
                 recentItem->setIcon(icon);
                 m_thumbnailMap[recentFileUrlPath] = recentItem->icon();
