@@ -32,12 +32,13 @@
 class KisRotateCanvasAction::Private
 {
 public:
-    Private() : angleDrift(0), previousAngle(0) {}
+    Private() : previousAngle(0) {}
 
     Shortcut mode;
-    qreal angleDrift;
 
     qreal previousAngle;
+    qreal startRotation;
+    qreal previousRotation;
 };
 
 
@@ -92,11 +93,10 @@ void KisRotateCanvasAction::begin(int shortcut, QEvent *event)
 
     switch(shortcut) {
         case RotateModeShortcut:
-            d->mode = (Shortcut)shortcut;
-            break;
         case DiscreteRotateModeShortcut:
             d->mode = (Shortcut)shortcut;
-            d->angleDrift = 0;
+            d->startRotation = inputManager()->canvas()->rotationAngle();
+            d->previousRotation = 0;
             break;
         case RotateLeftShortcut:
             canvasController->rotateCanvasLeft15();
@@ -110,29 +110,35 @@ void KisRotateCanvasAction::begin(int shortcut, QEvent *event)
     }
 }
 
-void KisRotateCanvasAction::cursorMoved(const QPointF &lastPos, const QPointF &pos)
+void KisRotateCanvasAction::cursorMovedAbsolute(const QPointF &startPos, const QPointF &pos)
 {
     const KisCoordinatesConverter *converter = inputManager()->canvas()->coordinatesConverter();
-    QPointF centerPoint = converter->flakeToWidget(converter->flakeCenterPoint());
-    QPointF oldPoint = lastPos - centerPoint;
-    QPointF newPoint = pos - centerPoint;
+    const QPointF centerPoint = converter->flakeToWidget(converter->flakeCenterPoint());
+    const QPointF startPoint = startPos - centerPoint;
+    const QPointF newPoint = pos - centerPoint;
 
-    qreal oldAngle = atan2(oldPoint.y(), oldPoint.x());
-    qreal newAngle = atan2(newPoint.y(), newPoint.x());
+    const qreal oldAngle = atan2(startPoint.y(), startPoint.x());
+    const qreal newAngle = atan2(newPoint.y(), newPoint.x());
 
-    qreal angle = (180 / M_PI) * (newAngle - oldAngle);
+    qreal newRotation = (180 / M_PI) * (newAngle - oldAngle);
 
     if (d->mode == DiscreteRotateModeShortcut) {
         const qreal angleStep = 15;
-        qreal initialAngle = inputManager()->canvas()->rotationAngle();
-        qreal roundedAngle = qRound((initialAngle + angle + d->angleDrift) / angleStep) * angleStep - initialAngle;
-        d->angleDrift += angle - roundedAngle;
-        angle = roundedAngle;
+
+        // avoid jumps at the beginning of the rotation action
+        if (qAbs(newRotation) > 0.5 * angleStep) {
+            const qreal currentCanvasRotation = converter->rotationAngle();
+            const qreal desiredOffset = newRotation - d->previousRotation;
+            newRotation = qRound((currentCanvasRotation + desiredOffset) / angleStep) * angleStep - currentCanvasRotation + d->previousRotation;
+        } else {
+            newRotation = d->previousRotation;
+        }
     }
 
     KisCanvasController *canvasController =
         dynamic_cast<KisCanvasController*>(inputManager()->canvas()->canvasController());
-    canvasController->rotateCanvas(angle);
+    canvasController->rotateCanvas(newRotation - d->previousRotation);
+    d->previousRotation = newRotation;
 }
 
 void KisRotateCanvasAction::inputEvent(QEvent* event)
@@ -154,21 +160,50 @@ void KisRotateCanvasAction::inputEvent(QEvent* event)
             if (touchEvent->touchPoints().count() != 2)
                 break;
 
-            QPointF p0 = touchEvent->touchPoints().at(0).pos();
-            QPointF p1 = touchEvent->touchPoints().at(1).pos();
+            QTouchEvent::TouchPoint tp0 = touchEvent->touchPoints().at(0);
+            QTouchEvent::TouchPoint tp1 = touchEvent->touchPoints().at(1);
+
+            if (tp0.state() == Qt::TouchPointReleased ||
+                tp1.state() == Qt::TouchPointReleased)
+            {
+                // Workaround: on some devices, the coordinates of TouchPoints
+                // in state TouchPointReleased are not reliable, and can
+                // "jump" by a significant distance. So we just stop handling
+                // the rotation as soon as the user's finger leaves the tablet.
+                break;
+            }
+
+            QPointF p0 = tp0.pos();
+            QPointF p1 = tp1.pos();
+
+            if ((p0-p1).manhattanLength() < 10)
+            {
+                // The TouchPoints are too close together. Don't update the
+                // rotation as the angle will likely be off. This also deals
+                // with a glitch where a newly pressed TouchPoint incorrectly
+                // reports the existing TouchPoint's coordinates instead of its
+                // own.
+                break;
+            }
 
             // high school (y2 - y1) / (x2 - x1)
             QPointF slope = p1 - p0;
             qreal newAngle = atan2(slope.y(), slope.x());
-
-            if (!d->previousAngle)
-            {
-                d->previousAngle = newAngle;
-                return;
-            }
-
             qreal delta = (180 / M_PI) * (newAngle - d->previousAngle);
 
+            // Workaround: So, TouchPoint coordinates are not 100% reliable.
+            // Freshly pressed TouchPoints are sometimes not accurate for
+            // multiple events. So if the computed rotation angle is out of
+            // whack, we just ignore it: it probably means the previous
+            // position was off.
+            if (qAbs(delta) > 15) {
+                // TouchPoint coordinates tend to converge toward correct
+                // values, so assume the previous angle was off but this one is
+                // better and should be the new basis for the next event.
+                d->previousAngle = newAngle;
+                break;
+            }
+
             KisCanvas2 *canvas = inputManager()->canvas();
             KisCanvasController *controller = static_cast<KisCanvasController*>(canvas->canvasController());
             controller->rotateCanvas(delta);
@@ -187,3 +222,9 @@ KisInputActionGroup KisRotateCanvasAction::inputActionGroup(int shortcut) const
     Q_UNUSED(shortcut);
     return ViewTransformActionGroup;
 }
+
+bool KisRotateCanvasAction::supportsHiResInputEvents(int shortcut) const
+{
+    Q_UNUSED(shortcut);
+    return true;
+}
