@@ -87,7 +87,7 @@ public:
     /**
      * Called by KisToolProxy when the primary is no longer possible
      * to be started now, e.g. when its modifiers and released. The
-     * tool is supposed revert all the preparetions it has doen in
+     * tool is supposed to revert all the preparations it has done in
      * activatePrimaryAction().
      */
     virtual void deactivatePrimaryAction();
@@ -135,6 +135,7 @@ public:
     enum ToolAction {
         Primary,
         AlternateChangeSize,
+        AlternateChangeSizeSnap,
         AlternatePickFgNode,
         AlternatePickBgNode,
         AlternatePickFgImage,
@@ -150,6 +151,7 @@ public:
     // So these can basically be thought of as aliases to ctrl+click, etc.
     enum AlternateAction {
         ChangeSize = AlternateChangeSize, // Default: Shift+Left click
+        ChangeSizeSnap = AlternateChangeSizeSnap, // Default: Shift+Z+Left click
         PickFgNode = AlternatePickFgNode, // Default: Ctrl+Alt+Left click
         PickBgNode = AlternatePickBgNode, // Default: Ctrl+Alt+Right click
         PickFgImage = AlternatePickFgImage, // Default: Ctrl+Left click
@@ -178,7 +180,7 @@ public:
     virtual void continueAlternateAction(KoPointerEvent *event, AlternateAction action);
     virtual void endAlternateAction(KoPointerEvent *event, AlternateAction action);
     virtual void beginAlternateDoubleClickAction(KoPointerEvent *event, AlternateAction action);
-
+    virtual bool alternateActionSupportsHiResEvents(AlternateAction action) const;
 
     void mousePressEvent(KoPointerEvent *event) override;
     void mouseDoubleClickEvent(KoPointerEvent *event) override;
@@ -212,6 +214,8 @@ protected:
     QPointF convertToPixelCoord(KoPointerEvent *e);
     QPointF convertToPixelCoord(const QPointF& pt);
 
+    QPointF convertToPixelCoordAndAlignOnWidget(const QPointF& pt);
+
     QPointF convertToPixelCoordAndSnap(KoPointerEvent *e, const QPointF &offset = QPointF(), bool useModifiers = true);
     QPointF convertToPixelCoordAndSnap(const QPointF& pt, const QPointF &offset = QPointF());
 
