@@ -5,7 +5,8 @@
  *
  *  This library is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; version 2.1 of the License.
+ *  the Free Software Foundation; version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,9 +33,14 @@ class PerspectiveAssistant : public KisAbstractPerspectiveGrid, public KisPainti
     Q_OBJECT
 public:
     PerspectiveAssistant(QObject * parent = 0);
+    KisPaintingAssistantSP clone(QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> &handleMap) const override;
+
     QPointF adjustPosition(const QPointF& point, const QPointF& strokeBegin) override;
+    void setAdjustedBrushPosition(const QPointF position) override;
+    void setFollowBrushPosition(bool follow) override;
     void endStroke() override;
-    QPointF buttonPosition() const override;
+
+    QPointF getEditorPosition() const override;
     int numHandles() const override { return 4; }
     void drawAssistant(QPainter& gc, const QRectF& updateRect, const KisCoordinatesConverter* converter, bool cached = true,KisCanvas2* canvas=0, bool assistantVisible=true, bool previewVisible=true) override;
 
@@ -52,6 +58,7 @@ private:
     bool quad(QPolygonF& out) const;
     // finds the transform from perspective coordinates (a unit square) to the document
     bool getTransform(QPolygonF& polyOut, QTransform& transformOut) const;
+    explicit PerspectiveAssistant(const PerspectiveAssistant &rhs, QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> &handleMap);
 
     // which direction to snap to (in transformed coordinates)
     QLineF m_snapLine;
@@ -59,7 +66,12 @@ private:
     mutable QTransform m_cachedTransform;
     mutable QPolygonF m_cachedPolygon;
     mutable QPointF m_cachedPoints[4];
-    mutable bool m_cacheValid;
+    mutable bool m_cacheValid {false};
+    // Needed to make sure that when we are in the middle of a brush stroke, the
+    // guides follow the brush position, not the cursor position.
+    bool m_followBrushPosition;
+    bool m_adjustedPositionValid;
+    QPointF m_adjustedBrushPosition;
 };
 
 class PerspectiveAssistantFactory : public KisPaintingAssistantFactory
