@@ -24,6 +24,7 @@
 
 #include <kis_brush_size_option.h>
 #include <kis_deform_option.h>
+#include "kis_algebra_2d.h"
 
 #include <time.h>
 
@@ -40,11 +41,11 @@ enum DeformModes {GROW, SHRINK, SWIRL_CW, SWIRL_CCW, MOVE, LENS_IN, LENS_OUT, DE
 class DeformProperties
 {
 public:
-    int action;
-    qreal deformAmount;
-    bool useBilinear;
-    bool useCounter;
-    bool useOldData;
+    int action {0};
+    qreal deformAmount {0.0};
+    bool useBilinear {false};
+    bool useCounter {false};
+    bool useOldData {false};
 };
 
 
@@ -53,10 +54,11 @@ class DeformBase
 public:
     DeformBase() {}
     virtual ~DeformBase() {}
-    virtual void transform(qreal * x, qreal * y, qreal distance) {
+    virtual void transform(qreal * x, qreal * y, qreal distance, KisRandomSourceSP randomSource) {
         Q_UNUSED(x);
         Q_UNUSED(y);
         Q_UNUSED(distance);
+        Q_UNUSED(randomSource);
     }
 };
 
@@ -71,14 +73,15 @@ public:
     qreal factor() {
         return m_factor;
     }
-    void transform(qreal* x, qreal* y, qreal distance) override {
-        qreal scaleFactor = (1.0 - distance) * m_factor + distance;
+    void transform(qreal* x, qreal* y, qreal distance, KisRandomSourceSP randomSource) override {
+        Q_UNUSED(randomSource);
+        qreal scaleFactor = KisAlgebra2D::signPZ(m_factor) * (qAbs((1.0 - distance) * m_factor) + distance);
         *x = *x / scaleFactor;
         *y = *y / scaleFactor;
     }
 
 private:
-    qreal m_factor;
+    qreal m_factor {0.0};
 };
 
 /// Inverse weighted rotation - swirlCW&&swirlCWW
@@ -89,7 +92,8 @@ public:
     void setAlpha(qreal alpha) {
         m_alpha = alpha;
     }
-    void transform(qreal* maskX, qreal* maskY, qreal distance) override {
+    void transform(qreal* maskX, qreal* maskY, qreal distance, KisRandomSourceSP randomSource) override {
+        Q_UNUSED(randomSource);
         distance = 1.0 - distance;
         qreal rotX = cos(-m_alpha * distance) * (*maskX) - sin(-m_alpha * distance) * (*maskY);
         qreal rotY = sin(-m_alpha * distance) * (*maskX) + cos(-m_alpha * distance) * (*maskY);
@@ -99,7 +103,7 @@ public:
     }
 
 private:
-    qreal m_alpha;
+    qreal m_alpha {0.0};
 };
 
 /// Inverse move
@@ -113,15 +117,16 @@ public:
         m_dx = dx;
         m_dy = dy;
     }
-    void transform(qreal* maskX, qreal* maskY, qreal distance) override {
+    void transform(qreal* maskX, qreal* maskY, qreal distance, KisRandomSourceSP randomSource) override {
+        Q_UNUSED(randomSource);
         *maskX -= m_dx * m_factor * (1.0 - distance);
         *maskY -= m_dy * m_factor * (1.0 - distance);
     }
 
 private:
-    qreal m_dx;
-    qreal m_dy;
-    qreal m_factor;
+    qreal m_dx {0.0};
+    qreal m_dy {0.0};
+    qreal m_factor {0.0};
 };
 
 /// Inverse lens distortion
@@ -140,8 +145,9 @@ public:
         m_out = out;
     }
 
-    void transform(qreal* maskX, qreal* maskY, qreal distance) override {
+    void transform(qreal* maskX, qreal* maskY, qreal distance, KisRandomSourceSP randomSource) override {
         Q_UNUSED(distance);
+        Q_UNUSED(randomSource);
         //normalize
         qreal normX = *maskX / m_maxX;
         qreal normY = *maskY / m_maxY;
@@ -163,9 +169,9 @@ public:
     }
 
 private:
-    qreal m_k1, m_k2;
-    qreal m_maxX, m_maxY;
-    bool m_out;
+    qreal m_k1 {0.0}, m_k2 {0.0};
+    qreal m_maxX {0.0}, m_maxY {0.0};
+    bool m_out {false};
 };
 
 /// Randomly disturb the pixels
@@ -173,22 +179,21 @@ class DeformColor : public DeformBase
 {
 public:
     DeformColor() {
-        srand48(time(0));
     }
 
     void setFactor(qreal factor) {
         m_factor = factor;
     }
-    void transform(qreal* x, qreal* y, qreal distance) override {
+    void transform(qreal* x, qreal* y, qreal distance, KisRandomSourceSP randomSource) override {
         Q_UNUSED(distance);
-        qreal randomX = m_factor * ((drand48() * 2.0) - 1.0);
-        qreal randomY = m_factor * ((drand48() * 2.0) - 1.0);
+        qreal randomX = m_factor * ((randomSource->generateNormalized() * 2.0) - 1.0);
+        qreal randomY = m_factor * ((randomSource->generateNormalized() * 2.0) - 1.0);
         *x += randomX;
         *y += randomY;
     }
 
 private:
-    qreal m_factor;
+    qreal m_factor {0.0};
 };
 
 
@@ -202,11 +207,11 @@ public:
     DeformBrush();
     ~DeformBrush();
 
-    KisFixedPaintDeviceSP paintMask(KisFixedPaintDeviceSP dab, KisPaintDeviceSP layer,
+    KisFixedPaintDeviceSP paintMask(KisFixedPaintDeviceSP dab, KisPaintDeviceSP layer, KisRandomSourceSP randomSource,
                                     qreal scale, qreal rotation, QPointF pos,
                                     qreal subPixelX, qreal subPixelY, int dabX, int dabY);
 
-    void setSizeProperties(BrushSizeOption * properties) {
+    void setSizeProperties(KisBrushSizeOptionProperties * properties) {
         m_sizeProperties = properties;
     }
     void setProperties(DeformOption * properties) {
@@ -236,16 +241,16 @@ private:
 
 private:
     KisRandomSubAccessorSP m_srcAcc;
-    bool m_firstPaint;
-    qreal m_prevX, m_prevY;
-    int m_counter;
+    bool m_firstPaint {false};
+    qreal m_prevX {0.0}, m_prevY {0.0};
+    int m_counter {1}; // taken from the constructor
 
     QRectF m_maskRect;
 
-    DeformBase * m_deformAction;
+    DeformBase * m_deformAction {0};
 
-    DeformOption * m_properties;
-    BrushSizeOption * m_sizeProperties;
+    DeformOption * m_properties {0};
+    KisBrushSizeOptionProperties * m_sizeProperties {0};
 };
 
 
